import json
from hashlib import md5

from django.db.models import CharField, Value
from django.db.models.functions import Cast, Concat, NullIf

from arkindex.project.tools import RTrimChr

__default_farm = None


def get_default_farm():
    return None


def hash_object(object):
    object_json = json.dumps(object, sort_keys=True).encode("utf-8")
    return md5(object_json).hexdigest()


def annotate_image_url(field="image"):
    """
    Build the image URL by concatenating the server's URL to the image's path
    Server URLs might end in a slash, but not all the time,
    so we strip any trailing slashes and add our own.
    This uses NullIf as this concatenation will return '/' if there is no zone on the element.
    """
    return NullIf(
        Concat(
            RTrimChr(f"{field}__server__url", Value("/")),
            Value("/"),
            Cast(f"{field}__path", CharField()),
        ),
        Value("/"),
    )
